# -*- coding: utf-8 -*-
"""
    KPortageTray - A set of basic operations for Portage.
    
    Copyright (C) 2010  Ronan Arraes Jardim Chagas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 3 as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """

##\file KPT_MainWindow.py
##\brief Class to construct the main window.
## Class to construct and handle all the events related to the main window and the system tray icon.
##\author Ronan Arraes Jardim Chagas

"""
Imports
"""
import sys
import os
import dbus
import time
from PyQt4 import QtCore, QtGui
from PyKDE4.kdecore import *
from PyKDE4.kdeui import *
from PyKDE4.kparts import *

# Local imports
from KPT_Definitions import *
from KPTUI_MainWindow import Ui_KPTUI_MainWindow
from KPT_Portage import *
from KPT_dbus import *

class KPT_MainWindow(QtGui.QMainWindow):
	"""
	Main class of the application.
	It is used to construct the main window.
	"""

	## Variable to store the information to be displayed at About Dialog.
	_aboutData = None

	## Instace of the class KPT_dbus.
	_dbus = None

	## Default timeout (msecs) for messages displayed by DBUS Notify system.
	_defaultTimeout = 8000

	## Instance of the class KPT_Portage.
	_portage = None

	## KPT_MainWindow constructor.
	def __init__(self, parent=None):
		super(KPT_MainWindow, self).__init__(parent)

		# Create a instance of the class KPT_Portage.
		self._portage = KPT_Portage()

		# Create a instante of the class to handle DBUS.
		self._dbus = KPT_dbus()

		# Create thread's instances.

		## Thread to sync portage tree.
		self.threadSync = KPT_PortageSync()
		## Thread to upgrade world set.
		self.threadUpgrade = KPT_PortageUpgrade()
		## Thread to check upgrades.
		self.threadCheckUpgrades = KPT_PortageCheckUpgrades()
		self.threadCheckUpgrades.setThreadSync(self.threadSync)
		## Thread to upgrade portage.
		self.threadUpgradePortage = KPT_PortageUpgradePortage()
		## Thread to execute etc-update.
		self.threadEtcUpdate = KPT_PortageEtcUpdate()
		## Thread to execute revdep-rebuild.
		self.threadRevdepRebuild = KPT_PortageRevdepRebuild()
		## Thread to execute dep-clean.
		self.threadDepClean = KPT_PortageDepClean()

		# Connect the signals.
		self.connect(self.threadSync, QtCore.SIGNAL("finished()"), self.syncFinished)
		self.connect(self.threadSync, QtCore.SIGNAL("terminated()"), self.syncFinished)
		self.connect(self.threadCheckUpgrades, QtCore.SIGNAL("finished()"), self.checkUpgradesFinished)
		self.connect(self.threadCheckUpgrades, QtCore.SIGNAL("terminated()"), self.checkUpgradesFinished)

		# Create the UI of the main window.
		self.ui = Ui_KPTUI_MainWindow()           
		self.ui.setupUi(self)

		# Create the actions.
		self.createActions()
		# Create the System Tray Icon.
		self.createTrayIcon()
		# Set Icons.
		self.trayIcon.setIcon(QtGui.QIcon(KPORTAGETRAY_INSTALLDIR+"/images/gentoo.svg"))
		self.setWindowIcon(QtGui.QIcon(KPORTAGETRAY_INSTALLDIR+"/images/gentoo.svg"))
		# Show system tray icon.
		self.trayIcon.show()

		# Load Konsole KPART.
		konsoleFactory = KLibLoader.self().factory("libkonsolepart")
		konsolePart = konsoleFactory.create(self, "KonsolePart")
		self.ui.konsoleLayout.addWidget(konsolePart.widget())
		konsolePart.openUrl(KUrl.fromPath(os.environ['HOME']))

		# Update text boxes
		KPT_MainWindow.setInfoTextBoxes(self)
		self.ui.editAvailableUpgrades.setPlainText(i18n("Upgrade verification wasn't performed yet.\n"
								"Execute it using the Command tab or by pressing CTRL-C."))
		# Set the main window visible.
		self.setVisible(True)

	#
	# Events
	#

	## Close event.
	#  \param event Event.
	def closeEvent(self, event):
		if self.trayIcon.isVisible():
			QtGui.QMessageBox.information(self, i18n("Systray"),
					i18n("The program will keep running in the system "
						 "tray. To terminate the program, choose "
						 "<b>Quit</b> in the context menu of the system "
						 "tray icon, <b>Quit</b> option at <b>File</b> menu "
						 "or <b>Quit</b> button at <b>Command</b> tab."))
			self.hide()
			event.ignore()

	## Icon activated event.
	#  \param reason Event call reason.
	#  If the reason is double-click, the status will be changed from
	#  visible to hidden or hidden to visible.
	def iconActivated(self, reason):
		if reason == QtGui.QSystemTrayIcon.DoubleClick:
			self.setVisible(self.isHidden())

	## Change the visibility of the main window.
	#  \param visible True makes the window visible and False hide it
	def setVisible(self, visible):
		self.minimizeAction.setEnabled(visible)
		self.restoreAction.setEnabled(self.isMaximized() or not visible)
		super(KPT_MainWindow, self).setVisible(visible)

	#
	# Functions
	#

	## Shows the about dialog.
	def aboutDialog(self):
		aboutDialog = KAboutApplicationDialog(self._aboutData, self.parent())
		aboutDialog.exec_()

	## Create the actions of the Main Window and the System Tray Icon.
	def createActions(self):
		# Menu bar
		QtCore.QObject.connect(self.ui.actionQuit,QtCore.SIGNAL("triggered()"), QtGui.qApp.quit)

		QtCore.QObject.connect(self.ui.actionSyncPortageTree,QtCore.SIGNAL("triggered()"),self.sync)
		QtCore.QObject.connect(self.ui.actionUpgradeWorld,QtCore.SIGNAL("triggered()"),self.upgrade)
		QtCore.QObject.connect(self.ui.actionCheckUpgrades,QtCore.SIGNAL("triggered()"),self.checkUpgrades)
		QtCore.QObject.connect(self.ui.actionUpgradePortage,QtCore.SIGNAL("triggered()"),self.upgradePortage)
		QtCore.QObject.connect(self.ui.actionEtcUpdate,QtCore.SIGNAL("triggered()"),self.etcUpdate)
		QtCore.QObject.connect(self.ui.actionDepClean,QtCore.SIGNAL("triggered()"),self.depClean)
		QtCore.QObject.connect(self.ui.actionRevdepRebuild,QtCore.SIGNAL("triggered()"),self.revdepRebuild)

		QtCore.QObject.connect(self.ui.actionAbout,QtCore.SIGNAL("triggered()"),self.aboutDialog)

		# Buttons
		QtCore.QObject.connect(self.ui.buttonQuit,QtCore.SIGNAL("clicked()"),QtGui.qApp.quit)
		
		QtCore.QObject.connect(self.ui.buttonSync,QtCore.SIGNAL("clicked()"),self.sync)
		QtCore.QObject.connect(self.ui.buttonUpgrade,QtCore.SIGNAL("clicked()"),self.upgrade)
		QtCore.QObject.connect(self.ui.buttonCheckUpgrades,QtCore.SIGNAL("clicked()"),self.checkUpgrades)
		QtCore.QObject.connect(self.ui.buttonUpgradePortage,QtCore.SIGNAL("clicked()"),self.upgradePortage)
		QtCore.QObject.connect(self.ui.buttonEtcUpdate,QtCore.SIGNAL("clicked()"),self.etcUpdate)
		QtCore.QObject.connect(self.ui.buttonDepClean,QtCore.SIGNAL("clicked()"),self.depClean)
		QtCore.QObject.connect(self.ui.buttonRevdepRebuild,QtCore.SIGNAL("clicked()"),self.revdepRebuild)

		QtCore.QObject.connect(self.ui.buttonGenInfoUpdate,QtCore.SIGNAL("clicked()"),self.setInfoTextBoxes)
		QtCore.QObject.connect(self.ui.buttonCompilingUpdate,QtCore.SIGNAL("clicked()"),self.setCompilingTextBox)

		# System tray

		## Sync portage tree action.
		self.syncPortageTreeAction = QtGui.QAction(i18n("&Sync Portage Tree"), self)
		self.syncPortageTreeAction.triggered.connect(self.sync)

		## Upgrade world action.
		self.upgradeAction = QtGui.QAction(i18n("&Upgrade World"), self)
		self.upgradeAction.triggered.connect(self.upgrade)

		## Verify upgrades action.
		self.checkUpgradesAction = QtGui.QAction(i18n("&Check Upgrades"), self)
		self.checkUpgradesAction.triggered.connect(self.checkUpgrades)

		## Upgrade portage action.
		self.upgradePortageAction = QtGui.QAction(i18n("Upgrade &Portage"), self)
		self.upgradePortageAction.triggered.connect(self.upgradePortage)

		## Etc-Update action.
		self.etcUpdateAction = QtGui.QAction(i18n("&Etc-Update"), self)
		self.etcUpdateAction.triggered.connect(self.etcUpdate)

		## Dep-Clean action.
		self.depCleanAction = QtGui.QAction(i18n("&Dep-Clean"), self)
		self.depCleanAction.triggered.connect(self.depClean)

		## Revdep-Rebuild action.
		self.revdepRebuildAction = QtGui.QAction(i18n("&Revdep-Rebuild"), self)
		self.revdepRebuildAction.triggered.connect(self.revdepRebuild)

		## Minimize action.
		self.minimizeAction = QtGui.QAction(i18n("Mi&nimize"), self)
		self.minimizeAction.triggered.connect(self.hide)

		## Restore action.
		self.restoreAction = QtGui.QAction(i18n("Res&tore"), self)
		self.restoreAction.triggered.connect(self.showNormal)

		## Quit action.
		self.quitAction = QtGui.QAction(i18n("&Quit"), self)
		self.quitAction.triggered.connect(QtGui.qApp.quit)

	## Create the system tray icon.
	def createTrayIcon(self):
		## Context menu for system tray icon.
		self.trayIconMenu = QtGui.QMenu(self)
		self.trayIconMenu.addAction(self.syncPortageTreeAction)
		self.trayIconMenu.addAction(self.upgradeAction)
		self.trayIconMenu.addAction(self.checkUpgradesAction)
		self.trayIconMenu.addAction(self.upgradePortageAction)
		self.trayIconMenu.addAction(self.etcUpdateAction)
		self.trayIconMenu.addAction(self.revdepRebuildAction)
		self.trayIconMenu.addAction(self.depCleanAction)
		self.trayIconMenu.addSeparator()
		self.trayIconMenu.addAction(self.minimizeAction)
		self.trayIconMenu.addAction(self.restoreAction)
		self.trayIconMenu.addAction(self.quitAction)
		## System tray icon.
		self.trayIcon = QtGui.QSystemTrayIcon(self)
		self.trayIcon.setContextMenu(self.trayIconMenu)
		# Activated action of trayIcon.
		self.trayIcon.activated.connect(self.iconActivated)

	## Set the information to be displayed at About dialog.
	#  \param aboutData KAboutData class that contains the information to be displayer at About Dialog.
	def setAboutData(self, aboutData):
		KPT_MainWindow._aboutData = aboutData

	## Update compiling text box.
	def setCompilingTextBox(self):
		self.ui.editCompiling.setPlainText(QtCore.QString(self._portage.genlopText()))

	## Update text boxes.
	def setInfoTextBoxes(self):
		systemNameProcess = QtCore.QProcess()
		systemNameProcess.start('uname -a')
		emergeInfoProcess = QtCore.QProcess()
		emergeInfoProcess.start('emerge --info')
		systemNameProcess.waitForFinished()
		systemName = QtCore.QString(systemNameProcess.readAll()).split('\n')[0]
		emergeInfoProcess.waitForFinished()
		emergeInfo = QtCore.QString(emergeInfoProcess.readAll()).split('\n')

		for line in emergeInfo:
			if 'Portage' in line:
				portageVersion = line.split(' ')[1]
				break

		for line in emergeInfo:
			if 'Timestamp' in line:
				lastSync = line.replace('Timestamp of tree: ','')
				break

		self.ui.editPortageVersion.setText(portageVersion)
		self.ui.editSystemName.setText(systemName)
		self.ui.editLastSync.setText(lastSync)

	#
	# Actions
	#

	## Check if there are upgrades available.
	def checkUpgrades(self):
		# Check if there are another verification running.
		if(self.threadCheckUpgrades.isRunning()):
			self._dbus.notify(unicode(i18n("Another upgrades verification process is running.\nWait for it to be finished.")),
					unicode(i18n("Error!")), 'services')
			self.ui.statusbar.showMessage(i18n("Another upgrades verification process is running.\nWait for it to be finished."))
		else:
			# First, silent sync the tree.
			self.syncSilent()
			# Start the process to check upgrades.
			self.threadCheckUpgrades.start()

			# Show pertinent messages.
			self._dbus.notify(unicode(i18n("The system is verifying if there are upgrades available.")),
						unicode(i18n("Checking upgrades...")), 'services', self._defaultTimeout)
			self.ui.statusbar.showMessage(i18n("The system is verifying if there are upgrades available."))
			self.ui.editAvailableUpgrades.setPlainText(i18n("The system is verifying if there are upgrades available."))

	## Process to be executed after the finish of the threadCheckUpgrades.
	def checkUpgradesFinished(self):
		# Get the number of upgrades and process STDOUT.
		numUpgrades = self.threadCheckUpgrades.numUpgrades
		processStdout = self.threadCheckUpgrades.processStdout

		# Verify is there was an error.
		if(numUpgrades == -1):
			textInfo = QtCore.QString(i18n("It was detected an error that needs to be checked.\n"
							"Verify <B>Available Upgrades</B> tab for more details."))
			textAvailableUpgradesTab = i18n("It was detected an error that needs to be checked.\n"
							"Verify the entire output below for more details.\n")
			textAvailableUpgradesTab.append(processStdout)
		else:
			# Verify if there isn't upgrades.
			if(numUpgrades == 0):
				textInfo = QtCore.QString(i18n("The system is up-to-date."))
				textAvailableUpgradesTab = QtCore.QString(textInfo)
			else:
				textInfo = QtCore.QString(i18n("Number of packages to be installed: ")+unicode(numUpgrades)+"\n")
				textInfoHtml = QtCore.QString(textInfo)
				
				# Verify if there are a new portage version.
				if(self.threadCheckUpgrades.newPortageVersion):
					textInfo.append(i18n("\nA newer Portage version is available.\n"
							"It is highly recommended to upgrade portage before any package.\n"
							"It can be done using the option Upgrade Portage.\n\n"))
					textInfoHtml.append(i18n("\n<B>A newer Portage version is available.</B>\n"
	                                                        "It is highly recommended to upgrade portage before any package.\n"
	                                                        "It can be done using the option <B>Upgrade Portage</B>.\n\n"))

				# Display the packages that will be upgraded only if the number is less or equal 15.
				if(numUpgrades <= 15):
					textInfo.append(processStdout)
					textInfoHtml.append(processStdout)
					textAvailableUpgradesTab = QtCore.QString(textInfo)
				else:
					textInfoHtml.append(i18n("For a complete list, check <B>Available Upgrades<B> tab."))
					textAvailableUpgradesTab = QtCore.QString(processStdout)
				textAvailableUpgradesTab.append(i18n("\n\nTo install these packages, use Upgrade command at Command tab or just type CTRL-U."))


		# Text to be displayed by DBUS Notify.
		self._dbus.notify(unicode(textInfoHtml), unicode(i18n("Result of upgrades verification")), 'services')
		# Text to be displayed at status bar.
		self.ui.statusbar.showMessage(i18n("The available upgrades check process is completed, check Available Upgrades for details."))
		# Updates Available Upgrades tab
		self.ui.editAvailableUpgrades.setPlainText(textAvailableUpgradesTab)

	## Dep-Clean.
	def depClean(self):
		self.threadDepClean.start()
		self._dbus.notify(unicode(i18n("A console was opened to continue the dep-clean process.<BR><BR>"
					"<B>CAUTION:</B> It can <B>BREAK</B> your system!<BR>"
					"Look carefully the packages that will be deleted to avoid mistakes!<BR>"
					"It is also recommended to run a <B>system upgrade</B> before it and run <B>revdep-rebuild</B> after it.<BR>")),
				unicode(i18n("Dep-Clean Started")), 'services')
		self.ui.statusbar.showMessage(i18n("A console was opened to continue the dep-clean process."))

	## Etc-Update.
	def etcUpdate(self):
		self.threadEtcUpdate.start()
		self._dbus.notify(unicode(i18n("A console was opened to continue the etc-update process.")),
				unicode(i18n("Etc-Update Started")), 'services', self._defaultTimeout)
		self.ui.statusbar.showMessage(i18n("A console was opened to continue the etc-update process."))

	## Revdep-Rebuild.
	def revdepRebuild(self):
		self.threadRevdepRebuild.start()
		self._dbus.notify(unicode(i18n("A console was opened to continue the revdep-rebuild process.")),
				unicode(i18n("Revdep-Rebuild Started")), 'services', self._defaultTimeout)
		self.ui.statusbar.showMessage(i18n("A console was opened to continue the revdep-rebuild process."))

	## Sync portage tree.
	def sync(self):
		# Verify if the sync process is running.
		if(self.threadSync.isRunning()):
			self._dbus.notify(unicode(i18n("Another sync process is running.\nWait for it to be finished.")),
					unicode(i18n("Error!")), 'services')
			self.ui.statusbar.showMessage(i18n("Another sync process is running.\nWait for it to be finished."))
		else:
			# Start the sync process.
			self.threadSync.start()
			# Show pertinent information.
			self._dbus.notify(unicode(i18n("The sync process was started at background.\nWait for it to be completed.")),
					unicode(i18n("Sync Started")), 'services', self._defaultTimeout)
			self.ui.statusbar.showMessage(i18n("The sync process was started at background.\nWait for it to be completed."))

	## Sync portage tree without showing any messages.
	def syncSilent(self):
		# Verify if the sync process is running.
		if(self.threadSync.isRunning()):
			# In this case, just wait the finish of the process.
			while(self.threadSync.isRunning()):
				time.sleep(2)
		else:
			# Start the sync process.
			self.threadSync.start()

	## Process to be executed after a tree sync.
	def syncFinished(self):
		# Update text boxes.
		self.setInfoTextBoxes()
		# Display pertinent information.
		self._dbus.notify(unicode(i18n("The sync process was finished.\nCheck the timestamp to verify if it was successful.")),
				unicode(i18n("Sync Finished")), 'services', self._defaultTimeout)
		self.ui.statusbar.showMessage(i18n("The sync process was finished.\nCheck the timestamp to verify if it was successful."))

	## Upgrade world.
	def upgrade(self):
		self.threadUpgrade.start()
		self._dbus.notify(unicode(i18n("A console was opened to continue the upgrade process.")),
				unicode(i18n("Upgrade Started")), 'services', self._defaultTimeout)
		self.ui.statusbar.showMessage(i18n("A console was opened to continue the upgrade process."))
	
	## Upgrade world.
	def upgradePortage(self):
		self.threadUpgradePortage.start()
		self._dbus.notify(unicode(i18n("A console was opened to continue the Portage upgrade process.")),
				unicode(i18n("Portage Upgrade Started")), 'services', self._defaultTimeout)
		self.ui.statusbar.showMessage(i18n("A console was opened to continue the Portage upgrade process."))

